@@ -1,4 +1,5 @@
-vblrt<-function(len=NULL,age=NULL,group=NULL,error=1,select=1,Linf=c(NULL),K=c(NULL),t0=c(NULL),plottype=0,
+vblrt<-function(len=NULL,age=NULL,group=NULL,error=1,select=1,Linf=c(NULL),
+K=c(NULL),t0=c(NULL),plottype=0,
                 control=list(maxiter=10000,minFactor=1/1024,tol=1e-5)){
    if(is.null(len)) 
          stop ("len is missing") 
@@ -17,26 +18,36 @@ vblrt<-function(len=NULL,age=NULL,group=NULL,error=1,select=1,Linf=c(NULL),K=c(N
      	x<-as.data.frame(cbind(len,age,cat))
       index<-as.numeric(which(is.na(x),arr.ind=TRUE))[1]
      	if(!is.na(index)) x<-x[-index,]
-          storeLinf<-NULL; storeK<-NULL;storet0<-NULL
+         
+      if(select==1){ 
+           storeLinf<-NULL; storeK<-NULL;storet0<-NULL
            subcats<-sapply(2:ncol(x),function(y) list(x[,y]))
            subcats<-subcats[c(2:length(subcats),1)]
            subcats[[ngroups+1]]<-trunc(subcats[[ngroups+1]])
            g1<-aggregate(x[,1],subcats,mean)
            names(g1)[ngroups+1]<-"age"
+
            for(t in 2:ngroups){
              m1<-g1[g1[,t]==1,]
-             m1$x2<-NA
-             m1$x2[1:c(length(m1$x)-1)]<-m1$x[2:c(length(m1$x))]   
-             out1<-lm(x2~x,data=m1,subset=(!is.na(x2)))
-             KK<-abs(log(coef(out1)[2]))
+             temp<-m1[,c(ncol(m1)-1,ncol(m1))];names(temp)<-c("age","len")
+             m1$age<-m1$age+1
+             m1<-merge(m1,temp,by.x="age",by.y="age",all.x=TRUE,all.y=TRUE)
+             m1<-m1[-unique(which(is.na(m1),arr.ind=TRUE)[,1]),]   
+             out1<-lm(len~x,data=m1)
+             KK<-as.numeric(abs(log(coef(out1)[2])))
              LI<-coef(out1)[1]/(1-coef(out1)[2])
              dx1<-as.data.frame(cbind(LI-m1$x,m1$age));dx1<-dx1[dx1[,1]>0,]
              t0d<-(coef(lm(log(dx1[,1])~dx1[,2]))[1]-log(LI))/KK
              storeLinf<-c(storeLinf,as.numeric(LI))
              storeK<-c(storeK,as.numeric(KK))
              storet0<-c(storet0,as.numeric(t0d))
-           }
-      if(select==1){ 
+           } 
+           qL<-quantile(storeLinf,prob=c(0.1,0.90))
+           storeLinf<-ifelse(storeLinf<qL[1]|storeLinf>qL[2],median(storeLinf),storeLinf)
+           qK<-quantile(storeK,prob=c(0.1,0.90))
+           storeK<-ifelse(storeK<qK[1]|storeK>qK[2],median(storeK),storeK)
+           qt<-quantile(storet0,prob=c(0.1,0.90))
+           storet0<-ifelse(storet0<qt[1]|storet0>qt[2],median(storet0),storet0)
         Lparms<-c(max(storeLinf),-(max(storeLinf)-storeLinf))
         Kparms<-c(max(storeK),max(storeK)-storeK)
         indexs<-which(abs(storet0)==max(abs(storet0)))
@@ -266,5 +277,6 @@ vblrt<-function(len=NULL,age=NULL,group=NULL,error=1,select=1,Linf=c(NULL),K=c(N
             pch=16,cex = 1.5,col=primcolors[1:ngroups])
   }#plottype==2
 }#plottype>0
+par(mfrow=c(1,1))
   return(nlsout)
 }
